<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\FPA\Services\RelatCustomFeriasService;
use HCM\Util\HCMParameters;
use HCM\Util\Util;

class RelatCustomFeriasController extends Crud {

    CONST operatorLikeAll = 'LIKE_ALL';
    CONST operatorLike = 'LIKE';

    public function __construct (Manager $dataSourceManager, HCMParameters $hcmParameters, RelatCustomFeriasService $relatCustomFeriasService){
        $this->relatCustomFeriasService = $relatCustomFeriasService;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getRelatCustomFerias(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            set_time_limit(0);
            $filters = Util::removerCaracterePorcentagemString($request->getFilterCriteria()->getConditions());
            $parameters = array();
            $ignoraocorrecal = $idferiascalc = $ignoramodalidcalc = 'N';
            $fieldsUnset = array('P_DTMESCOMPETENC', 'P_NRORG', 'NRTPMODALIDCAL', 'NROCORRECAL', 'P_NRORG_PADRAO', 'NRTIPOHIERARQORG', 'CDMATRICULA', 'CDMATRICULAESOCIAL','NRESTRUTLEGAL', 'NRESTRUTGEREN', 'NRVINCULOM', 'QTDEDIASRESTANTE', 'NRVINCULOM_LIST', 'NRESTRUTGEREN_LIST', 'NRESTRUTLEGAL_LIST', 'IGNORAOCORRECAL', 'IDFERIASCALC');
            foreach ($filters as $key => $filter) {
                if (array_search($filter['columnName'], $fieldsUnset) !== false){
                    if($filter['columnName'] === 'NROCORRECAL'){
                        $filter['value'] = str_replace("%", "", $filter['value']);
                    }
                    if($filter['columnName'] === 'IGNORAOCORRECAL'){
                        $ignoraocorrecal = $filter['value'];
                    }
                    if($filter['columnName'] === 'IDFERIASCALC'){
                        $idferiascalc= $filter['value'];
                    }
                    unset($filters[$key]);
                }
                if($filter['operator'] == self::operatorLikeAll){
                    $filters[0]['operator'] = self::operatorLike;
                    $filters[0]['columnName'] = str_replace('|', '||', $filter['columnName']);
                }
                $parameters[$filter['columnName']] = $filter['value'];
            }
            if(array_key_exists('NRVINCULOM_LIST', $parameters) && !empty($parameters['NRVINCULOM_LIST'])){
                $vinculos = is_array($parameters['NRVINCULOM_LIST']) ? implode(',', $parameters['NRVINCULOM_LIST']) : $parameters['NRVINCULOM_LIST'];
                if(array_key_exists('NRVINCULOM', $parameters) && !empty($parameters['NRVINCULOM'])){
                    $parameters['NRVINCULOM'] .= ','.$vinculos; 
                }else{
                    $parameters['NRVINCULOM'] = $vinculos;
                }
                unset($parameters['NRVINCULOM_LIST']);
                unset($vinculos);
            }
            
            if(array_key_exists('NRESTRUTGEREN_LIST', $parameters) && !empty($parameters['NRESTRUTGEREN_LIST'])){
                $parameters['NRESTRUTGEREN'] = $parameters['NRESTRUTGEREN_LIST']; 
                unset($parameters['NRESTRUTGEREN_LIST']);
            }
            
            if(array_key_exists('NRESTRUTLEGAL_LIST', $parameters) && !empty($parameters['NRESTRUTLEGAL_LIST'])){
                $parameters['NRESTRUTLEGAL'] = $parameters['NRESTRUTLEGAL_LIST']; 
                unset($parameters['NRESTRUTLEGAL_LIST']);
            }
            
            unset($parameters['IGNORAOCORRECAL']);

            $result = $this->relatCustomFeriasService->getRelatCustomFerias($parameters, $filters, $this->hcmParameters->getOcorrenciaCalculo(), $this->hcmParameters->getNrModalidadeCalculo(), $ignoraocorrecal, $idferiascalc);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getRelatCustomFerias', $result));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }
} 