<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Repository\AlteSituacaoFuncionalRepository;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\Util;
use Setforms2\Shared\ORM\DateTime;
use HCM\Util\DateUtil;
use Teknisa\Model\Gestaopessoas\Entities\GpeAltesitufunc;
use Teknisa\Model\Gestaopessoas\Entities\GpeSitufuncm;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use HCM\Geral\Entity\GpealtesitufuncEntity;
use HCM\FPA\Entity\FpaferiasEntity;
use Doctrine\DBAL\Connection;

class AlteSituacaoFuncionalService {

    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters, SituacaoFuncionalService $situacaoFuncionalService,
                                VinculoServices $vinculoServices, AlteEstabilidadeService $alteEstabilidadeService,
                                AlteSituacaoFuncionalRepository $alteSituacaoFuncionalRepository, ESocialService $eSocialService) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->situacaoFuncionalService = $situacaoFuncionalService;
        $this->vinculoServices = $vinculoServices;
        $this->alteEstabilidadeService = $alteEstabilidadeService;
        $this->alteSituacaoFuncionalRepository = $alteSituacaoFuncionalRepository;
        $this->eSocialService = $eSocialService;
        $this->gpeAltesitufunc = Repositories::GPE_ALTESITUFUNC;
    }

    
    public function retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc) {
        try {
            return $this->alteSituacaoFuncionalRepository->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc);
        } catch (\Exception $e) {
            throw $e;
        }
    }    
    
    public function verificaPossibilidadeAlteracao($nrchave, $edicaoexclusao) {
        $edicaodisponivel = $this->eSocialService->verificaPossibilidadeAlteracao($nrchave, 'Afastamento', $edicaoexclusao);
         
        if ($edicaodisponivel === true){
            return true;
        } else {
            $message = Messages::evaluateMessage(MESSAGES::HCM00000_EDITAR_EXCLUIR_REGISTRO_AFASTAMENTO, $nrchave);
            return new \Zeedhi\Framework\DTO\Response\Message($message, DTO\Response\Message::TYPE_WARNING);
        }
    }
    
    public function beforeSave($nrorg, $nrvinculom, $competencCadastro, $dtinisitufunc) {
        try {
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom, "nrorg" => $nrorg));

            if($vinculom->getDtadmissaovinc()->format('Y/m/d') <= \HCM\Util\DateUtil::getDataDeString(substr($dtinisitufunc,0, 10),DateUtil::FORMATO_BRASILEIRO, true)->format('Y/m/d')){
                $this->vinculoServices->verificaHistoricoCompetencia($nrorg, $nrvinculom, $competencCadastro);
            }else{
                throw new \Exception('Data de Início da Situação Funcional anterior à admissão do vínculo', 15);
                return;
            }
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function afterDelete($nrorg, $nrvinculom, $dtinisitufunc) {
        try {
            $ultaltesitufunc = $this->alteSituacaoFuncionalRepository->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc);
            if (is_object($ultaltesitufunc)) {
                $this->connection->beginTransaction();
                $proxaltesitufunc = $this->alteSituacaoFuncionalRepository->retornaProximaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc);
                if(empty($proxaltesitufunc)) {
                    if($ultaltesitufunc->getNrsitufuncm() == 1) { /*em 'situação normal' a data fim deve ser excluída*/
                        $ultaltesitufunc->setDtfimsitufunc(null);
                        $ultaltesitufunc->setNrqtdiasitufunc(null);
                    }
                }else {
                    $nrqtdias = DateUtil::getQtdeDiasEntre($ultaltesitufunc->getDtinisitufunc(), $proxaltesitufunc->getDtinisitufunc());
                    $ultaltesitufunc->setNrqtdiasitufunc($nrqtdias);
                    $ultaltesitufunc->setDtfimsitufunc(DateUtil::subtraiIntervalo($proxaltesitufunc->getDtinisitufunc(), 1));
                }
                $this->entityManager->flush();
                $this->connection->commit();
                $this->atualizaSitufuncHistoricoVinculo($nrorg, $nrvinculom, $ultaltesitufunc->getNrsitufuncm());
            }
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }
    
    public function verificaEstabilidade($nrorg, $dtcompetencia, $nrvinculom, $dtrescisao) {
        
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $ultaltesitufunc = $this->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtrescisao);
        $fimaltesitufunc = null;
        if($ultaltesitufunc) {
            if ($ultaltesitufunc->getDtestabsitufunc()) {
                $fimaltesitufunc = $ultaltesitufunc->getDtestabsitufunc();
            } else {
                $fimaltesitufunc = $ultaltesitufunc->getDtfimsitufunc();
            }
        }
        //se houver data fim na situação funcional atual verifica se possui estabilidade.
        if ($fimaltesitufunc) {
            $nrsitufuncm = $ultaltesitufunc->getNrsitufuncm();
            
            $situfunch =  $this->situacaoFuncionalService->retornaUltimoHistoricoCompetencia($nrorg, $nrorgPadrao, $nrsitufuncm, $dtcompetencia);
            
            $diasestabilidade = $situfunch->getQtestabpostsitu();
            if ($diasestabilidade) {
                $fimaltesitufunc = DateUtil::adicionaIntervalo($fimaltesitufunc, $diasestabilidade);
            }
            $dtrescisao = \HCM\Util\DateUtil::getDataDeString(substr($dtrescisao,0, 10),DateUtil::FORMATO_BRASILEIRO, true);
            
            if (\HCM\Util\DateUtil::comparaDatas($dtrescisao, $fimaltesitufunc) && $situfunch->getNrsitufuncm() != GpeSitufuncm::SITUACAO_FUNCIONAL_NORMAL) {
                return true;
            }
        }
        // if(!$dtrescisao){
        //     $situfincdemitido = $this->entityManager->getRepository(REPOSITORIES::GPE_ALTESITUFUNC)->findOneBy(array('nrorg' => $nrorg, 'nrvinculom' => $nrvinculom, 'nrsitufuncm' => GpealtesitufuncEntity::SIT_FUNC_DEMITIDO));
        //     if($situfincdemitido){
        //         try{
        //             $this->connection->beginTransaction();
        //             $this->entityManager->remove($situfincdemitido);
        //             $this->entityManager->flush();
        //             $this->connection->commit();
        //         }catch (\Exception $e){
        //             $this->connection->rollBack();
        //             throw $e;
        //         }
                
        //     }
        // }
        return false;
    }
    
    public function verificaVinculoAfastado($nrorg, $nrvinculom, $dtrescisao) {
        try {
            $ultaltesitufunc = $this->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtrescisao);
            
            if ($ultaltesitufunc) {
                if($ultaltesitufunc->getNrsitufuncm() != GpeSitufuncm::SITUACAO_FUNCIONAL_NORMAL &&
                   $ultaltesitufunc->getNrsitufuncm() != GpeSitufuncm::SITUACAO_FUNCIONAL_ATESTADO) {
                    if ($ultaltesitufunc->getDtfimsitufunc()) {
                        //se houver data fim na situação funcional atual verifica se o fim é anterior à data informada.
                        $fimaltesitufunc = $ultaltesitufunc->getDtfimsitufunc();
                        $fimaltesitufunc = \HCM\Util\DateUtil::truncateString($fimaltesitufunc);
                        $fimaltesitufunc = \HCM\Util\DateUtil::getDataDeString($fimaltesitufunc);
                        if (\HCM\Util\DateUtil::comparaDatas($fimaltesitufunc, $dtrescisao)) {
                            return false;
                        }
                    }
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function filtraVinculos($nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgerenlist, $nrvinculomlist) {
        try {
            // Filtra vínculos
            $arrayVinculos = array();
            if (empty($nrvinculomlist)) {
                if (empty($nrestrutgerenlist)) {
                    $arrayVinculos = $this->vinculoServices->retornaVinculosCompetencia($nrorg, $dtmescompetenc, false, $nrestrutlegal, null, 'ESTRUTURA');
                } else {
                    foreach ($nrestrutgerenlist as $nrestrutgeren) {
                        $vinculos = $this->vinculoServices->retornaVinculosCompetencia($nrorg, $dtmescompetenc, false, $nrestrutlegal, $nrestrutgeren, 'ESTRUTURA');
                        $arrayVinculos = array_merge($arrayVinculos, $vinculos);
                    }
                }
            } else {
                $arrayVinculos = $this->vinculoServices->retornaVinculosCompetencia($nrorg, $dtmescompetenc, false, $nrvinculomlist, null, 'LISTA_VINCULO');
            }
            return $arrayVinculos;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function cadastroAutomatico(Response $response, $nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgerenlist, $nrvinculomlist, $dtinisitufunc, $dtfimsitufunc, $nrsitufuncm, $nrproxsitufunc, $motivoAfastamento, $qntDias, $dsObs) {
        try {
            $cadastrados = array();
            $vinculos = $this->filtraVinculos($nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgerenlist, $nrvinculomlist);
            foreach ($vinculos as $vinculo) {
                // Verifica se existe situação funcional posterior à nova
                $proxsitufunc = $this->alteSituacaoFuncionalRepository->retornaProximaSituacaoPorDatainicio($nrorg, $vinculo['NRVINCULOM'], $dtinisitufunc);
                if (is_object($proxsitufunc)) {
                    //@todo MsgCalculo_CADAUTOSITUFUNC
                    /*\MsgCalculo::getInstance()->adicionaMensagem(
                        'Vínculo '.$vinculo->getNrvinculom().'-'.$vinculo->getNmvinculom().' possui situação funcional posterior (Início:'.$situfuncpost->getDtinisitufunc().')',
                        \MsgCalculo::ERRO
                    );*/
                    $mensagem = Messages::evaluateMessage(Messages::HCM00000_GPE_ALTESITUFUNC_VINC_POSSUI_POSTERIOR, array($vinculo['NRVINCULOM'], $vinculo['NMVINCULOM'], $proxsitufunc->getDtinisitufunc()));
                    $response->addMessage(new Response\Message($mensagem));
                } else {
                    // Caso não exista histórico do vínculo na competência, cria um novo
                    $this->vinculoServices->verificaHistoricoCompetencia($nrorg, $vinculo['NRVINCULOM']);

                    // Insere nova situação funcional
                    $dtinicio = DateUtil::getDataDeString($dtinisitufunc, DateUtil::FORMATO_BRASILEIRO, true);
                    if ($dtfimsitufunc) {
                        $dtfim = DateUtil::getDataDeString($dtfimsitufunc, DateUtil::FORMATO_BRASILEIRO, true);
                    } else {
                        $dtfim = null;
                    }
                    
                    $this->nova($nrorg, $vinculo['NRVINCULOM'], $nrsitufuncm, $dtinicio, $motivoAfastamento, null, null, $dtfim, null, null, null, $nrproxsitufunc, $qntDias, $dsObs);
                    $this->entityManager->flush();
                    // Atualiza o histórico do vínculo
                    $this->atualizaSitufuncHistoricoVinculo($nrorg, $vinculo['NRVINCULOM'], $nrsitufuncm);

                    // Atualiza quantidade de dias e data final da situação funcional anterior à nova
                    $this->verifDtUltSitufunc($nrorg, $nrsitufuncm, $vinculo['NRVINCULOM'], $dtinisitufunc, $nrproxsitufunc);
                    $cadastrados[] = $vinculo['NRVINCULOM'];
                    
                    // Adiciona a próxima situação se houver 
                    if ($nrproxsitufunc) {
                        $proxsituacao = $this->verifProxSitufunc($nrorg, $vinculo['NRVINCULOM'], $dtinisitufunc, $dtfimsitufunc, $nrproxsitufunc);
                        if ($proxsituacao) {
                            // Atualiza o histórico do vínculo com a próxima situação
                            $this->atualizaSitufuncHistoricoVinculo($nrorg, $vinculo['NRVINCULOM'], $nrproxsitufunc);
                        }
                    }
                    
                }
            }

            if (empty($cadastrados)) {
                return Response\Notification::TYPE_ERROR;
            } elseif (count($cadastrados) == count($vinculos)) {
                return Response\Notification::TYPE_SUCCESS;
            } else {
                return Response\Notification::TYPE_ALERT;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function exclusaoAutomatica(Response $response, $nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgerenlist, $nrvinculomlist, $dtinisitufunc, $dtfimsitufunc, $nrsitufuncm, $motivoAfastamento) {
        try {
            $excluidos = array();
            $vinculos = $this->filtraVinculos($nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgerenlist, $nrvinculomlist);
            foreach ($vinculos as $vinculo) {
                // Verifica se existe a situação funcional, na data informada, para o vínculo
                $altesitufuncs = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)
                    ->findBy(array(
                        'nrorg' => $nrorg,
                        'nrvinculom' => $vinculo['NRVINCULOM'],
                        'dtinisitufunc' => DateUtil::getDataDeString($dtinisitufunc, DateUtil::FORMATO_BRASILEIRO, true)
                    ));
                if (empty($altesitufuncs)) {
                    //@todo MsgCalculo_EXCLAUTOSITUFUNC
                    /*\MsgCalculo::getInstance()->adicionaMensagem(
                        'Situação funcional não encontrada para o vínculo '.$vinculo->getNrvinculom().'-'.$vinculo->getNmvinculom(),
                        \MsgCalculo::ERRO
                    );*/
                    $mensagem = Messages::evaluateMessage(Messages::HCM00000_GPE_ALTESITUFUNC_VINC_N_ENCONTRADA, array($vinculo['NRVINCULOM'], $vinculo['NMVINCULOM']));
                    $response->addMessage(new Response\Message($mensagem));
                } else {
                    // Busca a situação funcional anterior
                    $datainicio = DateUtil::getDataDeString($dtinisitufunc, DateUtil::FORMATO_BRASILEIRO, true)->sub(new \DateInterval('P1D'))->format(DateUtil::FORMATO_BRASILEIRO);
                    $altesitufuncant = $this->alteSituacaoFuncionalRepository->retornaUltimaSituacaoPorDatainicio($nrorg, $vinculo['NRVINCULOM'], $datainicio);

                    if (!is_object($altesitufuncant)) {
                        //@todo MsgCalculo_EXCLAUTOSITUFUNC
                        /*\MsgCalculo::getInstance()->adicionaMensagem(
                            'Vínculo '.$vinculo->getNrvinculom().'-'.$vinculo->getNmvinculom().' sem situação funcional anterior',
                            \MsgCalculo::ERRO
                        );*/
                        $mensagem = Messages::evaluateMessage(Messages::HCM00000_GPE_ALTESITUFUNC_VINC_N_POSSUI_ANTERIOR, array($vinculo['NRVINCULOM'], $vinculo['NMVINCULOM']));
                        $response->addMessage(new Response\Message($mensagem));
                    } else {
                        // Busca a situação funcional posterior
                        $altesitufuncpost = $this->alteSituacaoFuncionalRepository->retornaProximaSituacaoPorDatainicio($nrorg, $vinculo['NRVINCULOM'], $dtinisitufunc);

                        if (is_object($altesitufuncpost)) {
                            //@todo MsgCalculo_EXCLAUTOSITUFUNC
                            /*\MsgCalculo::getInstance()->adicionaMensagem(
                                'Vínculo '.$vinculo->getNrvinculom().'-'.$vinculo->getNmvinculom().' possui situação funcional posterior (Início:'.$situfuncpost->getDtinisitufunc().')',
                                \MsgCalculo::ERRO
                            );*/
                            $mensagem = Messages::evaluateMessage(Messages::HCM00000_GPE_ALTESITUFUNC_VINC_POSSUI_POSTERIOR, array($vinculo['NRVINCULOM'], $vinculo['NMVINCULOM'], $altesitufuncpost->getDtinisitufunc()));
                            $response->addMessage(new Response\Message($mensagem));
                        } else {
                            // Exclui a situação funcional
                            foreach ($altesitufuncs as $altesitufunc) {
                                $this->entityManager->remove($altesitufunc);
                            }
                            $this->entityManager->flush();

                            // Limpa a quantidade de dias e a data final da situação funcional anterior
                            $altesitufuncant->setDtfimsitufunc(null);
                            $altesitufuncant->setNrqtdiasitufunc(null);
                            $this->entityManager->persist($altesitufuncant);

                            //Atualiza o histórico do vínculo
                            $this->atualizaSitufuncHistoricoVinculo($nrorg, $vinculo['NRVINCULOM'], $altesitufuncant->getNrsitufuncm());

                            $this->entityManager->flush();
                            $excluidos[] = $vinculo['NRVINCULOM'];
                        }
                    }
                }
            }

            if (empty($excluidos)) {
                return Response\Notification::TYPE_ERROR;
            } elseif (count($excluidos) == count($vinculos)) {
                return Response\Notification::TYPE_SUCCESS;
            } else {
                return Response\Notification::TYPE_ALERT;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida se já existe outra situação funcional para o vínculo com a mesma data
     *
     * @param $nrorg
     * @param $nrvinculom
     * @param $dtinisitufunc
     * @throws \Exception
     */
    public function validaExisteDataSituFunc($nrorg, $nrvinculom, $dtinisitufunc) {
        try {
            $altesitufuncs = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)
                ->findBy(array(
                    'nrorg' => $nrorg,
                    'nrvinculom' => $nrvinculom,
                    'dtinisitufunc' => DateUtil::getDataDeString($dtinisitufunc, DateUtil::FORMATO_BRASILEIRO, true)
                ));
            if (!empty($altesitufuncs)) {
                throw new \Exception(Messages::HCM00000_GER_ALTESITUFUNC_DATA_ALREADY_EXIST_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificAfastMesmoCid($nrorg, $nrsitufuncm, $nrvinculom, $dtinisitufunc, $dtfimsitufunc, $nrqtdiasitufunc, $cdtabecdi){
        try {
            $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $situfunch = $this->situacaoFuncionalService->retornaUltimoHistoricoCompetencia($nrorg, $nrorgpadrao, $nrsitufuncm, $dtmescompetenc);

            if (is_object($situfunch)) {
                if ($situfunch->getNrdiaretroativa() != null || $situfunch->getNrdiaretroativa() != 0) {

                    if ($nrsitufuncm == GpeSitufuncm::SITUACAO_FUNCIONAL_ATESTADO && $dtfimsitufunc) {
                        /*Armazena quantidade de dias de atestado a serem gravados na alteração*/
                        $diasatestadocorrente = $nrqtdiasitufunc;
                        /*O início do período de verificação será o fim do período de atestado 'acrescido' de -X dias retroativos*/
                        $nrdiasretroativos = $situfunch->getNrdiaretroativa() - 1;
                        $iniPeriodo = DateUtil::getDataDeString($dtfimsitufunc, DateUtil::FORMATO_BRASILEIRO, true);
                        $iniPeriodo->sub(new \DateInterval('P' . $nrdiasretroativos . 'D'));

                        /*O fim do período de verificação será o fim do período de atestado 'acrescido' de -$diasatestadoaux*/
                        $fimPeriodo = DateUtil::getDataDeString($dtfimsitufunc, DateUtil::FORMATO_BRASILEIRO, true);
                        $iniPeriodo->sub(new \DateInterval('P' . $diasatestadocorrente . 'D'));

                        /* Carrega todos os ATESTADOS do vinculo com o mesmo CID*/
                        $atestados = $this->alteSituacaoFuncionalRepository->retornaAlteSitFuncporVinculoNrcid($nrvinculom, GpeSitufuncm::SITUACAO_FUNCIONAL_ATESTADO, $cdtabecdi, $nrorg);
                        /* Verifica a intersecao dos dias de afastamento com o periodo de pagamento de folha*/
                        $afastamentos = $this->retornaDiasAfastamento($atestados, $iniPeriodo, $fimPeriodo, false);
                        /*Filtra a situação funcional atestado para contar e definir o limite de atestados*/
                        $filtra_situacao = array_filter($afastamentos, function ($filtra_situacao) {
                            return $filtra_situacao == GpeSitufuncm::SITUACAO_FUNCIONAL_ATESTADO;
                        });
                        $qtdAfast = count($filtra_situacao);
                        if ($qtdAfast + $diasatestadocorrente > $situfunch->getQtdiaslimitsitu()) {
                            $limitedias = ($qtdAfast + $diasatestadocorrente) - $situfunch->getQtdiaslimitsitu();
                            if ($limitedias > 0) {
                                $novaData = DateUtil::getDataDeString($dtfimsitufunc);
                                $novaData->sub(new \DateInterval('P' . $limitedias . 'D'));
                                $novaData = $novaData->format(DateUtil::FORMATO_BRASILEIRO);

                                $message = Messages::evaluateMessage(
                                    Messages::HCM00000_GPE_ALTESITUFUNC_QTD_ATESTADOS_CID_LIMITE,
                                    array($cdtabecdi, $situfunch->getNrdiaretroativa(), $situfunch->getQtdiaslimitsitu(), $novaData)
                                );

                                return array(
                                    'NRQTDIASITUFUNC' => $limitedias,
                                    'DTFIMSITUFUNC' => $novaData,
                                    'WARNING' => $message
                                );
                            } else {
                                $message = Messages::evaluateMessage(
                                    Messages::HCM00000_GPE_ALTESITUFUNC_QTD_ATESTADOS_CID_LIMITE_ERROR,
                                    $cdtabecdi,
                                    $cdtabecdi,
                                    $situfunch->getNrdiaretroativa(),
                                    $situfunch->getQtdiaslimitsitu()
                                );
                                throw new \Exception($message, Messages::ERROR_MESSAGE_CODE);
                            }
                        }
                    }

                    if ($nrsitufuncm == GpeSitufuncm::SITUACAO_FUNCIONAL_DOENCA_MENOR_15) {
                        /*O início do período de verificação será o inicio da situação a corrente -X dias retroativos*/
                        $iniPeriodo = DateUtil::getDataDeString($dtinisitufunc);
                        $iniPeriodo->sub(new \DateInterval('P' . $situfunch->getNrdiaretroativa() . 'D'));
                        /*O Fim do período de verificação será o inicio da situação a corrente -1 dia*/
                        $fimPeriodo = DateUtil::getDataDeString($dtinisitufunc);
                        $fimPeriodo->sub(new \DateInterval('P1D'));
                        /* Carrega todos as situações do vinculo com o mesmo CID*/
                        $altesitufuncs = $this->alteSituacaoFuncionalRepository->retornaAlteSitFuncporVinculoNrcid($nrvinculom, GpeSitufuncm::SITUACAO_FUNCIONAL_DOENCA_MAIOR_15, $cdtabecdi, $nrorg);
                        /* Verifica a intersecao dos dias de afastamento com o periodo de pagamento de folha*/
                        $afastamentos = $this->retornaDiasAfastamento($altesitufuncs, $iniPeriodo, $fimPeriodo, false);
                        if (in_array(GpeSitufuncm::SITUACAO_FUNCIONAL_DOENCA_MAIOR_15, $afastamentos)) {

                            $message = Messages::evaluateMessage(
                                Messages::HCM00000_GPE_ALTESITUFUNC_EXISTE_ATUALIZADA_AFASTAMENTO,
                                $cdtabecdi
                            );

                            return array(
                                'NRSITUFUNCM' => GpeSitufuncm::SITUACAO_FUNCIONAL_DOENCA_MAIOR_15,
                                'WARNING'     => $message
                            );
                        }
                    }

                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica se a situfunc anterior possui data final.
     * Caso nao possua seta com o dia anterior do inicio da nova situfunc.
     *
     * @throws \Exception
     */
    public function verifDtUltSitufunc($nrorg, $nrsitufuncm, $nrvinculom, $dtinisitufunc, $nrproxsitufunc = null){
        try {
            if ($nrsitufuncm == GpeSitufuncm::SITUACAO_FUNCIONAL_ATESTADO && !$nrproxsitufunc) {
                return;
            }
            $data = \HCM\Util\DateUtil::getDataDeString(substr($dtinisitufunc, 0, 10), 'd/m/Y', true);
            $data = \HCM\Util\DateUtil::subtraiIntervalo($data, 1);
            $datainicio = \HCM\Util\DateUtil::truncateString($data);
            $ultaltesitufunc = $this->alteSituacaoFuncionalRepository->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $datainicio);
            if (is_object($ultaltesitufunc)) {
                if ($ultaltesitufunc->getDtfimsitufunc() == '') {
                    $iniSit = \HCM\Util\DateUtil::truncateString($ultaltesitufunc->getDtinisitufunc());
                    $iniSit = \HCM\Util\DateUtil::getDataDeString($iniSit, 'd/m/Y', true);
                    $qtdDias = \HCM\Util\DateUtil::getQtdeDiasEntre($iniSit, $data);
                    $qtdDias++;

                    $ultaltesitufunc->setNrqtdiasitufunc($qtdDias);
                    $ultaltesitufunc->setDtfimsitufunc($data);

                    $this->entityManager->persist($ultaltesitufunc);
                    $this->entityManager->flush($ultaltesitufunc);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Cadastra uma proxima situ func se houver uma data fim e uma proxima situação funcinal setada.
     *
     * @throws \Exception
     */
    public function verifProxSitufunc($nrorg, $nrvinculom, $dtinisitufunc, $dtfimsitufunc, $nrproxsitufunc){
        try {
            $proxsitufunc = $this->alteSituacaoFuncionalRepository->retornaProximaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc, $nrproxsitufunc);
            if ($dtfimsitufunc && $nrproxsitufunc && !$proxsitufunc) {
                $dtfimsitufunc = \HCM\Util\DateUtil::getDataDeString($dtfimsitufunc, 'd/m/Y', true);
                $dtfimsitufunc = \HCM\Util\DateUtil::adicionaIntervalo($dtfimsitufunc, 1);
                $this->nova($nrorg, $nrvinculom, $nrproxsitufunc, $dtfimsitufunc);
                $this->entityManager->flush();
                return true;
            }
            return false;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraEstabilidadeSituFunc($nrorg, $nrvinculom, $nrsitufuncm, $dtinisitufunc, $nrdiasitufunc, $dtfimsitufunc, $dtestabsitufunc)
    {
        try {
            $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $situfunch = $this->situacaoFuncionalService->retornaUltimoHistoricoCompetencia($nrorg, $nrorgpadrao, $nrsitufuncm, $dtmescompetenc);
            if($situfunch){
                if ($situfunch->getQtestabpostsitu()) {
                    if ($dtestabsitufunc) {
                        $dataEstab = DateUtil::getDataDeString($dtestabsitufunc);
                    } elseif ($dtfimsitufunc) {
                        $dataEstab = DateUtil::getDataDeString($dtfimsitufunc);
                        $dataEstab->add(new \DateInterval('P1D'));
                    } elseif ($nrdiasitufunc) {
                        $dataEstab = DateUtil::getDataDeString($dtinisitufunc);
                        $dataEstab->add(new \DateInterval('P' . $nrdiasitufunc . 'D'));
                    } elseif ($situfunch->getQtdiaslimitsitu()) {
                        $nrdiaslimitsitu = $situfunch->getQtdiaslimitsitu();
                        $dataEstab = DateUtil::getDataDeString($dtinisitufunc);
                        $dataEstab->add(new \DateInterval('P' . $nrdiaslimitsitu . 'D'));
                    } else {
                        $dataEstab = null;
                    }
    
                    if (!is_null($dataEstab)) {
                        $dtiniestabvinc = $dataEstab->format('d/m/Y');
                        $nrdiasestab = $situfunch->getQtestabpostsitu();
                        $dataEstab->add(new \DateInterval('P' . $nrdiasestab . 'D'));
                        $dtfimestabvinc = $dataEstab->format('d/m/Y');
    
                        return $this->alteEstabilidadeService->geraEstabilidadeVinculo($nrorg, $nrvinculom, $dtiniestabvinc, $dtfimestabvinc);
                    }
                }
            }else{
                throw new \Exception('Situação funcional não encontrada.',10);
            }
            return false;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function nova($nrorg, $nrvinculom, $nrSituacaoFuncionario, $dtIniSituacaoFunc,
                         $motivoAfastamento = null, $cdDiagnostico = null, $codigoCDI = null,
                         $dtFimSituacaoFunc = null, $dtPreviretoSitu = null, $dtProrrSituacaoFunc = null,
                         $dtVerifSituacaoFunc = null, $nrProxSituFunc = null, $qntDias = null, $dsObs = null)
    {
        /** @var \Teknisa\Model\Gestaopessoas\Entities\GpeAltesitufunc $altesitufunc */
        
        $altesitufunc = new $this->gpeAltesitufunc;
        
        $altesitufunc->setNrorg($nrorg);
        $altesitufunc->setNrvinculom($nrvinculom);
        $altesitufunc->setNrsitufuncm($nrSituacaoFuncionario);
        $altesitufunc->setDtinisitufunc($dtIniSituacaoFunc);
        $altesitufunc->setNrqtdiasitufunc($qntDias);

        $altesitufunc->setNrmotivoafasta($motivoAfastamento);
        $altesitufunc->setCddiagnost($cdDiagnostico);
        $altesitufunc->setCdtabecdi($codigoCDI);
        $altesitufunc->setDtfimsitufunc($dtFimSituacaoFunc);
        $altesitufunc->setDtpreviretositu($dtPreviretoSitu);
        $altesitufunc->setDtprorrsitufunc($dtProrrSituacaoFunc);
        $altesitufunc->setDtverifsitufunc($dtVerifSituacaoFunc);
        $altesitufunc->setNrproxsitufunc($nrProxSituFunc);
        $altesitufunc->setDsObsAltSitFunc($dsObs);

        $altesitufunc->setNrorginclusao($nrorg);
        $altesitufunc->setDtinclusao(DateUtil::getDataAtual());
        $altesitufunc->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $altesitufunc->setNrorgultatu($nrorg);
        $altesitufunc->setDtultatu(DateUtil::getDataAtual());
        $altesitufunc->setCdoperultatu($this->hcmParameters->getCdoperador());

        $altesitufunc->setNraltesitufunc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESITUFUNC', 1, 12, true));

        $this->entityManager->persist($altesitufunc);
        return $altesitufunc;
    }

    /**
     * Atualiza número da situação funcional no histórico atual do vínculo
     *
     * @param $nrorg
     * @param $nrvinculom
     * @param $nrsitufuncm
     * @throws \Exception
     */
    public function atualizaSitufuncHistoricoVinculo($nrorg, $nrvinculom, $nrsitufuncm) {
        try {
            $gpevinculoh = $this->vinculoServices->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom);
            $gpevinculoh->setNrsitufuncm($nrsitufuncm);
            $gpevinculoh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $gpevinculoh->setCdoperultatu($this->hcmParameters->getCdoperador());
            $gpevinculoh->setDtultatu(DateUtil::getDataAtual());

            $this->entityManager->persist($gpevinculoh);
            $this->entityManager->flush($gpevinculoh);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /*=============MÉTODOS RETIRADOS DA CLASSE GpeAltesitufuncmEntity=============*/
    /**
     * Retorna dias de afastamento.
     *
     * @param GpeSitufuncm $situacao_funcional Situacao Funcional.
     * @param TkDateTime         $data_ini_per       Data de inicio de periodo.
     * @param TkDateTime         $data_fim_per       Data de fim de periodo.
     * @param boolean            $determina_prox_sf  Proxima situacao funcional.
     *
     * @return array
     *
     * @author alessandrobartels
     *         brunofeliciano - 01/09/2011 - correcao de complexidade ciclomatica.
     */
    public function retornaDiasAfastamento(
        $altesitufuncs, DateTime $data_ini_per, DateTime $data_fim_per,
        $determina_prox_sf
    ) {
        // rigel 23/02/2013 - alterados todos os for de 0 com <= para <
        $afastamento = array();

        //Quando a $data_ini_per e a $data_fim_per forem iguais o daysafter retorna 0       BORBA 10/07/2013
        //Utilizado também no método retornaDiasAfastamento na classe GpesitufuncmEntity para gerar afastamento
        //e no metodo carregaDiasUteis dentro de CalendarioLogic
        if($data_ini_per==$data_fim_per){
            $dias_periodo = 1;
        } else {
            $dias_periodo = DateUtil::getQtdeDiasEntre($data_ini_per,$data_fim_per);
        }

        /*o array é preenchido com -1 em todas as posições e onde a situação for normal ele preenche na
        posição o valor 1, onde houve afastamento é preenchido o número do afastamento para que seja gerado o evento
        correspondente  */
        for ($i=0; $i < $dias_periodo; $i++) {
            $afastamento[$i] = -1;
        }

        $persist = false;
        $data_fim_flag = true;
        $data_ini_aux = new DateTime();
        $data_fim_aux = new DateTime();
        $cont_alt_sits = (count($altesitufuncs));
        $cont_itera = 0;
        foreach ($altesitufuncs as $item) {
            $cont_itera++;
            $this->_verificaProxSituacaoFuncional(
                $determina_prox_sf, $item, $data_fim_per, $persist
            );

            $data_ini_aux = DateUtil::getDataDeString($item->getDtinisitufunc());

            if (DateUtil::comparaDatas($data_ini_aux,$data_fim_per,DateUtil::DEPOIS) === true) {
                // Afastamento comecou depois do periodo tratado.
                break;
            }

            /* @todo Verificar o tipo de retorno do item */

            $data_fim_aux = $this->_retornaDataFimSituacao($item->getDtfimsitufunc(), $data_fim_per, $data_fim_aux);

            // Dentro do periodo aquisitivo.
            if (((DateUtil::comparaDatas($data_ini_aux,$data_ini_per,DateUtil::DEPOIS) === true)
                    ||(DateUtil::comparaDatas($data_ini_aux,$data_ini_per,DateUtil::IGUAL) === true))
                &&((DateUtil::comparaDatas($data_ini_aux,$data_fim_per) === true)
                    ||(DateUtil::comparaDatas($data_ini_aux,$data_fim_per,DateUtil::IGUAL) === true))
            ) {
                /* Comecou e acabou durante.
                 * Conta dias de data_ini_aux ate data_fim_aux.
                 */
                $dia_ini = DateUtil::getQtdeDiasEntre($data_ini_per,$data_ini_aux) - 1;

                //Quando a data_ini_aux e a data_fim_aux forem iguais o daysafter retorna 0       BORBA 19/04/2013
                //Utilizado também no método retornaDiasAfastamento na classe GpesitufuncmEntity para gerar afastamento
                //e no metodo carregaDiasUteis dentro de CalendarioLogicpara geras dias periodo
                if($data_ini_aux==$data_fim_aux){
                    $qtd_dias = 1;
                } else {
                    $qtd_dias = DateUtil::getQtdeDiasEntre($data_ini_aux,$data_fim_aux);
                }
                // Ajustado para 0 quando negativo pois o array começa de 0 e nos demais casos trabalha com a posicao anterior;
                if ($dia_ini < 0){
                    $dia_ini = 0;
                }

                for ($i=0; $i < $qtd_dias; $i++) {
                    $afastamento[$i+$dia_ini] = (int)$item->getNrsitufuncm();
                }
            } elseif ((DateUtil::comparaDatas($data_ini_aux,$data_ini_per) === true)
                &&((DateUtil::comparaDatas($data_fim_aux,$data_fim_per) === true)
                    || (DateUtil::comparaDatas($data_fim_aux,$data_fim_per,DateUtil::IGUAL) === true))
                || (DateUtil::comparaDatas($data_fim_aux,$data_ini_per,DateUtil::IGUAL) === true)
            ) {
                if ((DateUtil::comparaDatas($data_fim_aux,$data_ini_per,DateUtil::DEPOIS) === true)
                    ||(DateUtil::comparaDatas($data_fim_aux,$data_ini_per,DateUtil::IGUAL) === true)
                ) {
                    /* Comecou antes e acabou durante.
                     * Conta dias do data_ini_per ate data_fim_aux.
                     */

                    //Quando a data_ini_aux e a data_fim_aux forem iguais o daysafter retorna 0       BORBA 19/04/2013
                    //Utilizado também no método retornaDiasAfastamento na classe GpesitufuncmEntity para gerar afastamento
                    //e no metodo carregaDiasUteis dentro de CalendarioLogicpara geras dias periodo
                    if($data_ini_per==$data_fim_aux){
                        $qtd_dias = 1;
                    } else {
                        $qtd_dias = DateUtil::getQtdeDiasEntre($data_ini_per,$data_fim_aux);
                    }

                    for ($i=0; $i < $qtd_dias; $i++) {
                        $afastamento[$i] = (int)$item->getNrsitufuncm();
                    }
                } /*else {
                    // Comecou e acabou antes.
                }*/
            }
        }

        if ($persist === true) {
            $this->persist();
        }

        return $afastamento;
    }

    /**
     * Verifica Proxima Situacao Funcional.
     *
     * @param boolean               $determina_prox_sf  Flag de Proxima Situacao Funcional.
     * @param Gpealtesitufunc       $item               Situacao Funcional.
     * @param DateTime              $data_fim_per       Data de fim de periodo.
     * @param boolean               &$persist           Flag para Persist.
     *
     * @return void
     *
     * @author brunofeliciano/alessandrobartels - 02/09/2011
     */
    private function _verificaProxSituacaoFuncional(
        $determina_prox_sf, GpeAltesitufunc $item, DateTime $data_fim_per, &$persist
    ) {
        if ($determina_prox_sf === true) {
            // So determina proxima situacao funcional se for calculo. Em caso de ferias nao determina.
            $proxSituacaoFuncional = $this->_determinaProximaSituacaoFuncional(
                $item, $data_fim_per
            );
            if ($proxSituacaoFuncional === true) {
                $persist = true;
            }
        }
    }

    /**
     * Determina a proxima situacao funcional.
     *
     * @param GpealtesitufuncEntity $item               Situacao Funcional.
     * @param TkDateTime            $data_fim_per       Data de fim de periodo.
     *
     * @return boolean
     *
     * @author alessandrobartels
     */
    private function _determinaProximaSituacaoFuncional(
        Gpealtesitufunc $item, DateTime $data_fim_per
    ) {
        $data_inicio_afastamento = DateUtil::getDataDeString($item->getDtinisitufunc());
        $data_fim_afastamento = $item->getDtfimsitufunc();
        $persist = false;
        if (empty($data_fim_afastamento) === true) {
            // Projeta data fim.
            $sit_func = $this->situacaoFuncionalService->retornaUltimoHistoricoCompetencia(
                $this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getNrorgPadrao(), $item->getNrsitufuncm(), $this->hcmParameters->getDtCompetencia()
            );

            $qtd_limite_afastamento = (int)$sit_func->getQtdiaslimitsitu();

            /* @todo testar se $qtd_limite_afastamento <= 0 (empty) */
            if (empty($qtd_limite_afastamento) === true
                ||($qtd_limite_afastamento === 0)
            ) {
                // Se nao existe limite na ha como determinar proxima situacao.
                return $persist;
            }

            $data_fim_afastamento = DateUtil::getDataDeString($data_inicio_afastamento);
            $data_fim_afastamento->add(new \DateInterval('P'.($qtd_limite_afastamento-1).'D'));
            if (DateUtil::comparaDatas($data_fim_afastamento,$data_fim_per) === true) {
                // Encerra afastamento.
                $item->setDtfimsitufunc($data_fim_afastamento);
                $persist = true;
            }

            /* Uma proxima situacao no registro de alteracao (Gpealtesitufunc) tem prioridade sobre
             * uma proxima situacao na definicao da situacao funcional (Gpesitufunc).
             */
            $prox_situacao_func = $item->getNrproxsitufunc();
            if ($prox_situacao_func === null) {
                $prox_situacao_func = $sit_func->getNrproxsitufunc();
            }

            if ($prox_situacao_func !== null) {
                $data_ini_afastamento_aux = DateUtil::getDataDeString($data_fim_afastamento);

                $novo_item = new $this->gpeAltesitufunc;
                $novo_item->setNraltesitufunc(rand(1, 10000));
                $novo_item->setNrorg($this->getNrorg());
                /*
                 * $item->setNrmotivoafasta();
                 * $item->setNrproxsitufunc();
                 * $item->setCdtabecdi();
                 * $item->setCddiagnost();
                 */
                $novo_item->setNrsitufuncm($prox_situacao_func);
                $novo_item->setNrvinculom($this->getNrvinculom());
                $novo_item->setDtinisitufunc($data_ini_afastamento_aux->addDay(1));
                /*
                 *           $item->setDtfimsitufunc();
                 *			$item->setDtverifsitufunc();
                 * 			$item->setDtpreviretositu();
                 *			$item->setDtprorrsitufunc();
                 */
                $novo_item->setDtinclusao(now());
                $novo_item->setDtultatu(now());
                $novo_item->setNrorginclusao($this->getNrorg());
                /*			$item->setCdoperinclusao(); */
                $novo_item->setNrorgultatu($this->getNrorg());
                /*			$item->setCdoperultatu(); */

                //$this->addItem($novo_item, $novo_item->getNraltesitufunc());

                $persist = true;
            }

            return $persist;
        }

    }

    /**
     * Retorna a Data Fim da Situacao.
     *
     * @param string     $data_fim     Data fim.
     * @param TkDateTime $data_fim_per Data fim do periodo.
     * @param TkDateTime $data_fim_aux Data fim auxiliar.
     *
     * @return TkDateTime
     *
     * @author brunofeliciano/alessandrobartels - 02/09/2011
     */
    private function _retornaDataFimSituacao($data_fim, DateTime $data_fim_per, DateTime $data_fim_aux) {
        $data_fim_flag = true;
        if ($data_fim != null) {
            $data_fim_aux = DateUtil::getDataDeString($data_fim);
        } else {
            $data_fim_flag = false;
        }

        if (($data_fim_flag === false)
            ||(DateUtil::comparaDatas($data_fim_aux,$data_fim_per,DateUtil::DEPOIS) === true)
        ) {
            $data_fim_aux = DateUtil::getDataDeString($data_fim_per);
        }

        return $data_fim_aux;
    }
    
    public function retornaProximaSituacaoPorDatainicio ($nrorg, $nrvinculom, $dtinisitufunc, $nrproxsitufunc, $nraltesitufunc) {
        if ($nrproxsitufunc !== ''){
            $altefunc = $this->alteSituacaoFuncionalRepository->retornaProximaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc, null, $nraltesitufunc);
            
            if (!$altefunc)
                return true;
            else 
                return false;
        } else 
            return true;
    }

    public function manipulaSituacaoFuncional($dtrescisaoatual,$nrorg,$nrvinculom,$dtIniSituacaoFunc,$acao,$nrmotivoafasta,
    $nrsitufunc = 1, $dtrescisaohist = null)
    {
        switch ($acao) {
            case 'INSERT':
                if (!$nrmotivoafasta) {
                    throw new \Exception('Selecione o motivo de afastamento.',10);
                }
                $this->nova($nrorg,$nrvinculom, $nrsitufunc, $dtIniSituacaoFunc);
                break;
            case 'ALTERA':
                return self::regraMudarDataRescisao($nrorg, $nrvinculom, $dtrescisaoatual, $nrmotivoafasta, $dtrescisaohist);
                break;
        }
    }
    
    public function regraMudarDataRescisao($nrorg, $nrvinculom, $dtrescisao, $nrmotivoafasta, $dtrescisaohist = null)
    {
        try {
            $this->entityManager->beginTransaction();
            $nrsitufunc = 13;
            $nr = null;
            if($dtrescisaohist){
                $dtultimodia = $dtrescisaohist;
            }else {
                $dtultimodia = $this->hcmParameters->getUltimoDiaMes();
            }
            $ultimaSituacao = $this->alteSituacaoFuncionalRepository->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtultimodia);
            if (!$ultimaSituacao) {
                throw new \Exception('Vinculo sem situacao funcional!');
            }
            
            if (!$dtrescisao && $ultimaSituacao->getNrsitufuncm() == 13) { // Situacao Demitido
                // Retirar rescisao
                $dtrescis = $ultimaSituacao->getDtinisitufunc();
                $this->entityManager->beginTransaction();
                $this->entityManager->remove($ultimaSituacao);
                $this->entityManager->flush();
                $this->entityManager->commit();
                $penultima = $this->alteSituacaoFuncionalRepository->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtultimodia);
                if($penultima && $penultima->getNrsitufuncm() == 1 && $penultima->getDtfimsitufunc() && $penultima->getDtfimsitufunc()->modify('+ 1 day')->format('d/m/Y') == $dtrescis->format('d/m/Y')){
                    $penultima->setDtfimsitufunc(null);
                }
            } else if ($dtrescisao) {
                // Alterar data de rescisao
                $dtrescisao = \HCM\Util\DateUtil::getDataDeString($dtrescisao,DateUtil::FORMATO_BRASILEIRO, true);
                
                if ($ultimaSituacao->getNrsitufuncm() == 13) {
                    $ultimaSituacao->setDtinisitufunc($dtrescisao);
                    $nr = $ultimaSituacao->getNrsitufuncm();
                } else {
                    // Cria nova situacao de rescisao
                    $altesitufunc = $this->nova($nrorg,$nrvinculom, $nrsitufunc, $dtrescisao, $nrmotivoafasta);
                    $dtrescisao = clone $dtrescisao;
                    $dtrescisao->modify('-1 day');
                    if(!$ultimaSituacao->getDtfimsitufunc()) {
                        $ultimaSituacao->setDtfimsitufunc($dtrescisao);
                    }
                    $nr = $altesitufunc->getNrsitufuncm();
                }
            }
            $this->entityManager->flush();
            $this->entityManager->commit();
            if($nr){
                return $nr;
            }
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function verificaPerdaPeriodoFerias($nrorg, $nrvinculom){
        error_reporting(E_ERROR | E_PARSE);
        $Periodosferias = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->findBy(array('nrorg' => $nrorg, 'nrvinculom' => $nrvinculom));
        
        $fpaFeriasEntity = new FpaferiasEntity($this->entityManager, $this->hcmParameters);
        $alteracao_situacao_funcional = new GpealtesitufuncEntity($this->entityManager, $this->hcmParameters);
        $alteracao_situacao_funcional->retornaAlteracaoSituacaoFuncional($nrorg, $nrvinculom, null);
        
        $dias_limite = 180;
        foreach($Periodosferias as $ferias){
            
            $diasAfastados = $fpaFeriasEntity->_totalizaDiasAfastamento($alteracao_situacao_funcional, $ferias->getDtiniaquisicao(),
                                                            $ferias->getDtfimaquisicao(), $dias_limite);
            if($diasAfastados >= $dias_limite){
                $ferias->setQtdiasmaxfe(0);
                $ferias->setQtdiasmaxabon(0);
                $ferias->setIdcontrolferias(FpaferiasEntity::IDCONTROLFERIAS_PERDIDO);
                $ferias->setDtultatu(DateUtil::getDataAtual());
                $this->entityManager->persist($ferias);
                $this->entityManager->flush($ferias);
            }
        }
    }
    
    public function atualizaGozoFerias($nrorg, $nrvinculom, $nrferias, $nrgozoferias, $dtFimGozo, $saldo) {
        try{
            $dtFimGozo = DateUtil::getDataDeString($dtFimGozo);
            $this->entityManager->beginTransaction();
            
            $ferias = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->findBy(array('nrorg' => $nrorg, 'nrvinculom' => $nrvinculom, 'nrferias' => $nrferias));
            $ferias = $ferias[0];
            $gozoFerias = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->findBy(array('nrorg' => $nrorg, 'nrgozoferias' => $nrgozoferias));
            $gozoFerias = $gozoFerias[0];
            
            $gozoFerias->setDtfimgozoferias($dtFimGozo);
            $ferias->setQtDiasSaldo($saldo);
            
            // set auditoria gozo de férias
            $gozoFerias->setDtultatu(DateUtil::getDataAtual());
            $gozoFerias->setCdoperultatu($this->hcmParameters->getCdoperador());
            $gozoFerias->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            
            // set auditoria férias
            $ferias->setDtultatu(DateUtil::getDataAtual());
            $ferias->setCdoperultatu($this->hcmParameters->getCdoperador());
            $ferias->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

            $this->entityManager->flush();
            $this->entityManager->commit();
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function verificaSaldoFerias($nrorg, $nrvinculom, $dtinicio) {
        $message = null;
        $saldo = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->getSaldoFerias($nrorg, $nrvinculom, $dtinicio);

        if ($saldo > 0) {
            $message = Messages::evaluateMessage(Messages::HCM90000_ESO_FERIAS_COM_SALDO, array($nrvinculom, $saldo));
        }
        
        if ($message)
            return $message;
        else 
            return null;
    }

} 